package it.lineacomune.pagopa.be.ws;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.ws.config.annotation.EnableWs;
import org.springframework.ws.config.annotation.WsConfigurerAdapter;
import org.springframework.ws.transport.http.MessageDispatcherServlet;
import org.springframework.ws.wsdl.wsdl11.DefaultWsdl11Definition;
import org.springframework.xml.validation.XmlValidator;
import org.springframework.xml.xsd.SimpleXsdSchema;
import org.springframework.xml.xsd.XsdSchema;
import org.springframework.xml.xsd.XsdSchemaCollection;

@EnableWs
@Configuration
public class WebServiceConfig extends WsConfigurerAdapter {
    @Bean
    public ServletRegistrationBean messageDispatcherServlet(ApplicationContext applicationContext) {
        MessageDispatcherServlet servlet = new MessageDispatcherServlet();
        servlet.setApplicationContext(applicationContext);
        servlet.setTransformWsdlLocations(true);
        return new ServletRegistrationBean(servlet, "/ws/*");
    }

    @Bean
    @Qualifier("comunicaEsitoPagamentiAttesiSchema")
    public XsdSchema comunicaEsitoPagamentiAttesiSchema() {
        return new SimpleXsdSchema(new ClassPathResource("comunicaEsitoPagamentiAttesiWs.xsd"));
    }

    /*
    @Bean
    @Qualifier("pagopaFe")
    public XsdSchema FeSchema() {
        return new SimpleXsdSchema(new ClassPathResource("pagopaFe.xsd"));
    }
    */

    @Bean(name = "esiti")
    public DefaultWsdl11Definition defaultWsdl11Definition(XsdSchema pagamentiAttesiSchema) {
        DefaultWsdl11Definition wsdl11Definition = new DefaultWsdl11Definition();
        wsdl11Definition.setPortTypeName("esitiPagamentiAttesiWSDL");
        wsdl11Definition.setLocationUri("/esiti");
        wsdl11Definition.setSchemaCollection(esitiSchemaCollection());
        return wsdl11Definition;
    }
    
    @Bean
    public XsdSchemaCollection esitiSchemaCollection() {
        return new XsdSchemaCollection() {

            public XsdSchema[] getXsdSchemas() {
                return new XsdSchema[]{comunicaEsitoPagamentiAttesiSchema()};
            }

            public XmlValidator createValidator() {
                throw new UnsupportedOperationException();
            }
        };
    }
}
