package it.lineacomune.pagopa.be.ws.pagopabe;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.PayloadRoot;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;

import it.lineacomune.pagopa.be.ws.endpoint.CodiceEsitoComunicazione;
import it.lineacomune.pagopa.be.ws.endpoint.PagamentiAttesiEsitoPagamentoRequest;
import it.lineacomune.pagopa.be.ws.endpoint.PagamentiAttesiEsitoPagamentoResponse;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoEsitiPagamenti;
import it.lineacomune.pagopa.core.codiciritorno.SuccessoGenerico;

@Endpoint
@ComponentScan
public class PagopabeEndpoint {

	private static final Logger logger = LogManager.getLogger(PagopabeEndpoint.class);

	private static final String NAMESPACE_URI = "it/lineacomune/pagopa/be/ws/endpoint";

	@PayloadRoot(namespace = NAMESPACE_URI, localPart = "pagamentiAttesiEsitoPagamentoRequest")
	@ResponsePayload
	public PagamentiAttesiEsitoPagamentoResponse pagamentiAttesiEsitoPagamentoRequest(
			@RequestPayload PagamentiAttesiEsitoPagamentoRequest request) {
		PagamentiAttesiEsitoPagamentoResponse response = new PagamentiAttesiEsitoPagamentoResponse();
		response.setCodiceServizio(request.getCodiceServizio());
		response.setEnte(request.getEnte());
		try {
			response.setEsitoComunicazione(CodiceEsitoComunicazione.SUCCESS);
			response.setDescrizioneErrore(SuccessoGenerico.SUCCESS.getValue());
		} catch (Exception e) {
			logger.error("Eccezione Generica durante il tentativo di inviare gli Esiti di Pagamento", e);
			response.setEsitoComunicazione(CodiceEsitoComunicazione.ERROR);
			response.setDescrizioneErrore(ErrorePagamentoAttesoEsitiPagamenti.ERR_BE_PA_ESI_003.getValue());
		} finally {
			return response;
		}
	}

}
