package it.lineacomune.pagopa.core.exception.pagamentiattesi;

import it.lineacomune.pagopa.core.codiciritorno.ErroreAnagraficaVersante;
import it.lineacomune.pagopa.core.codiciritorno.ErroreArticoloDebitorio;
import it.lineacomune.pagopa.core.codiciritorno.ErroreEmailRt;
import it.lineacomune.pagopa.core.codiciritorno.ErroreEnte;
import it.lineacomune.pagopa.core.codiciritorno.ErroreEsitoPagamento;
import it.lineacomune.pagopa.core.codiciritorno.ErroreGenerico;
import it.lineacomune.pagopa.core.codiciritorno.ErroreListaIuv;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAtteso;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoAnnulla;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoEsitiPagamenti;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoInserisci;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoModifica;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoRiattiva;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePagamentoAttesoSegnaPagato;
import it.lineacomune.pagopa.core.codiciritorno.ErrorePartitaDebitoria;
import it.lineacomune.pagopa.core.codiciritorno.ErroreRicevutaTelematica;
import it.lineacomune.pagopa.core.codiciritorno.ErroreRichiestaPagamentoTelematico;
import it.lineacomune.pagopa.core.codiciritorno.ErroreServizio;
import it.lineacomune.pagopa.core.codiciritorno.SuccessoGenerico;

public class PagoPaScambioException extends RuntimeException {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private Enum errore;
	private String codiceErrore;
	private String descrizioneErrore;
	private String iuv;
	private String idBoPagamentoAtteso;
	private String idBoPartitaDebitoria;
	private String codiceEntrataArticoloDebitorio;

	public PagoPaScambioException(Enum errore) {
		super(errore.name());
		buildErrore(errore);
	}

	private void buildErrore(Enum errore) {
		this.errore = errore;
		extractErrorePagoPa();
	}

	public PagoPaScambioException(Enum errore, String iuv) {
		super(errore.name());
		buildErrore(errore);
		this.iuv = iuv;
	}

	public PagoPaScambioException(Enum errore, String iuv, String idBoPagamentoAtteso) {
		super(errore.name());
		buildErrore(errore);
		buildDettaglioPagamentoAtteso(null, null, iuv, idBoPagamentoAtteso);
	}

	public PagoPaScambioException(Enum errore, String iuv, String idBoPagamentoAtteso, String idBoPartitaDebitoria) {
		super(errore.name());
		buildErrore(errore);
		buildDettaglioPartitaDebitoria(null, null, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria);
	}

	public PagoPaScambioException(Enum errore, String iuv, String idBoPagamentoAtteso, String idBoPartitaDebitoria,
			String codiceEntrataArticoloDebitorio) {
		super(errore.name());
		buildErrore(errore);
		buildDettaglioArticoloDebitorio(null, null, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria,
				codiceEntrataArticoloDebitorio);
	}

	public PagoPaScambioException(Throwable e, Enum errore) {
		super(e);
		extractPagoPaException(e, errore);
	}

	public PagoPaScambioException(Throwable e, Enum errore, String iuv) {
		super(e);
		extractPagoPaException(e, errore);
		this.iuv = iuv;
	}

	public PagoPaScambioException(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso) {
		super(e);
		buildDettaglioPagamentoAtteso(e, errore, iuv, idBoPagamentoAtteso);
	}

	public PagoPaScambioException(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso,
			String idBoPartitaDebitoria) {
		super(e);
		buildDettaglioPartitaDebitoria(e, errore, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria);
	}

	public PagoPaScambioException(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso,
			String idBoPartitaDebitoria, String codiceEntrataArticoloDebitorio) {
		super(e);
		buildDettaglioArticoloDebitorio(e, errore, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria,
				codiceEntrataArticoloDebitorio);
	}

	private void extractPagoPaException(Throwable e, Enum errore) {
		if (e instanceof PagoPaScambioException) {
			PagoPaScambioException eccezionePagoPa = (PagoPaScambioException) e;
			if (eccezionePagoPa.getErrore() != null) {
				this.errore = eccezionePagoPa.getErrore();
			}
		} else {
			this.errore = errore;
		}
		extractErrorePagoPa();
	}

	private void buildDettaglioPagamentoAtteso(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso) {
		if (e != null && errore != null) {
			extractPagoPaException(e, errore);
		}
		this.iuv = iuv;
		this.idBoPagamentoAtteso = idBoPagamentoAtteso;
	}

	private void buildDettaglioPartitaDebitoria(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso,
			String idBoPartitaDebitoria) {
		if (e != null && errore != null) {
			buildDettaglioPagamentoAtteso(e, errore, iuv, idBoPagamentoAtteso);
		} else {
			buildDettaglioPagamentoAtteso(null, null, iuv, idBoPagamentoAtteso);
		}
		this.idBoPartitaDebitoria = idBoPartitaDebitoria;
	}

	private void buildDettaglioArticoloDebitorio(Throwable e, Enum errore, String iuv, String idBoPagamentoAtteso,
			String idBoPartitaDebitoria, String codiceEntrataArticoloDebitorio) {
		if (e != null && errore != null) {
			buildDettaglioPartitaDebitoria(e, errore, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria);
		} else {
			buildDettaglioPartitaDebitoria(null, null, iuv, idBoPagamentoAtteso, idBoPartitaDebitoria);
		}
		this.codiceEntrataArticoloDebitorio = codiceEntrataArticoloDebitorio;
	}

	public String getCodiceErrore() {
		return codiceErrore;
	}

	public void setCodiceErrore(String codiceErrore) {
		this.codiceErrore = codiceErrore;
	}

	public String getIuv() {
		return iuv;
	}

	public void setIuv(String iuv) {
		this.iuv = iuv;
	}

	public String getIdBoPagamentoAtteso() {
		return idBoPagamentoAtteso;
	}

	public void setIdBoPagamentoAtteso(String idBoPagamentoAtteso) {
		this.idBoPagamentoAtteso = idBoPagamentoAtteso;
	}

	public Enum getErrore() {
		return errore;
	}

	public void setErrore(Enum errore) {
		this.errore = errore;
	}

	public String getDescrizioneErrore() {
		return descrizioneErrore;
	}

	public void setDescrizioneErrore(String descrizioneErrore) {
		this.descrizioneErrore = descrizioneErrore;
	}

	private void extractErrorePagoPa() {
		if (errore == null) {
			return;
		}
		if (errore instanceof ErroreAnagraficaVersante) {
			this.codiceErrore = ((ErroreAnagraficaVersante) errore).name();
			this.descrizioneErrore = ((ErroreAnagraficaVersante) errore).getValue();
		}
		if (errore instanceof ErroreArticoloDebitorio) {
			this.codiceErrore = ((ErroreArticoloDebitorio) errore).name();
			this.descrizioneErrore = ((ErroreArticoloDebitorio) errore).getValue();
		}
		if (errore instanceof ErroreEmailRt) {
			this.codiceErrore = ((ErroreEmailRt) errore).name();
			this.descrizioneErrore = ((ErroreEmailRt) errore).getValue();
		}
		if (errore instanceof ErroreEnte) {
			this.codiceErrore = ((ErroreEnte) errore).name();
			this.descrizioneErrore = ((ErroreEnte) errore).getValue();
		}
		if (errore instanceof ErroreEsitoPagamento) {
			this.codiceErrore = ((ErroreEsitoPagamento) errore).name();
			this.descrizioneErrore = ((ErroreEsitoPagamento) errore).getValue();
		}
		if (errore instanceof ErroreGenerico) {
			this.codiceErrore = ((ErroreGenerico) errore).name();
			this.descrizioneErrore = ((ErroreGenerico) errore).getValue();
		}
		if (errore instanceof ErroreListaIuv) {
			this.codiceErrore = ((ErroreListaIuv) errore).name();
			this.descrizioneErrore = ((ErroreListaIuv) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAtteso) {
			this.codiceErrore = ((ErrorePagamentoAtteso) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAtteso) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoAnnulla) {
			this.codiceErrore = ((ErrorePagamentoAttesoAnnulla) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoAnnulla) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoEsitiPagamenti) {
			this.codiceErrore = ((ErrorePagamentoAttesoEsitiPagamenti) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoEsitiPagamenti) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoInserisci) {
			this.codiceErrore = ((ErrorePagamentoAttesoInserisci) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoInserisci) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoModifica) {
			this.codiceErrore = ((ErrorePagamentoAttesoModifica) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoModifica) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoSegnaPagato) {
			this.codiceErrore = ((ErrorePagamentoAttesoSegnaPagato) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoSegnaPagato) errore).getValue();
		}
		if (errore instanceof ErrorePagamentoAttesoRiattiva) {
			this.codiceErrore = ((ErrorePagamentoAttesoRiattiva) errore).name();
			this.descrizioneErrore = ((ErrorePagamentoAttesoRiattiva) errore).getValue();
		}
		if (errore instanceof ErrorePartitaDebitoria) {
			this.codiceErrore = ((ErrorePartitaDebitoria) errore).name();
			this.descrizioneErrore = ((ErrorePartitaDebitoria) errore).getValue();
		}
		if (errore instanceof ErroreRicevutaTelematica) {
			this.codiceErrore = ((ErroreRicevutaTelematica) errore).name();
			this.descrizioneErrore = ((ErroreRicevutaTelematica) errore).getValue();
		}
		if (errore instanceof ErroreRichiestaPagamentoTelematico) {
			this.codiceErrore = ((ErroreRichiestaPagamentoTelematico) errore).name();
			this.descrizioneErrore = ((ErroreRichiestaPagamentoTelematico) errore).getValue();
		}
		if (errore instanceof ErroreServizio) {
			this.codiceErrore = ((ErroreServizio) errore).name();
			this.descrizioneErrore = ((ErroreServizio) errore).getValue();
		}
		if (errore instanceof SuccessoGenerico) {
			this.codiceErrore = ((SuccessoGenerico) errore).name();
			this.descrizioneErrore = ((SuccessoGenerico) errore).getValue();
		}
	}

	public String getIdBoPartitaDebitoria() {
		return idBoPartitaDebitoria;
	}

	public void setIdBoPartitaDebitoria(String idBoPartitaDebitoria) {
		this.idBoPartitaDebitoria = idBoPartitaDebitoria;
	}

	public String getCodiceEntrataArticoloDebitorio() {
		return codiceEntrataArticoloDebitorio;
	}

	public void setCodiceEntrataArticoloDebitorio(String codiceEntrataArticoloDebitorio) {
		this.codiceEntrataArticoloDebitorio = codiceEntrataArticoloDebitorio;
	}
}
